@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtMultimedia of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 package org.qtproject.qt.android.multimedia;
 
@@ -48,6 +12,7 @@ import java.io.FileInputStream;
 import android.content.Context;
 import android.media.MediaPlayer;
 import android.media.MediaFormat;
+import android.media.PlaybackParams;
 import android.media.AudioAttributes;
 import android.media.TimedText;
 import android.net.Uri;
@@ -759,4 +724,28 @@ public class QtAndroidMediaPlayer
             Log.w(TAG, exception);
         }
     }
+
+    public boolean setPlaybackRate(float rate)
+    {
+        PlaybackParams playbackParams = mMediaPlayer.getPlaybackParams();
+        playbackParams.setSpeed(rate);
+        // According to discussion under the patch from QTBUG-61115: At least with DirectShow
+        // and GStreamer, it changes both speed and pitch. (...) need to be consistent
+        if (rate != 0.0)
+            playbackParams.setPitch(Math.abs(rate));
+
+        try {
+            mMediaPlayer.setPlaybackParams(playbackParams);
+        } catch (IllegalStateException | IllegalArgumentException e) {
+            Log.e(TAG, "Cannot set playback rate " + rate + " :" + e.toString());
+            return false;
+        }
+
+        if ((mState & State.Started) == 0 && mMediaPlayer.isPlaying()) {
+            setState(State.Started);
+            startProgressWatcher();
+        }
+
+        return true;
+    }
 }
