@@ -15,7 +15,9 @@
 #include <qpair.h>
 #include <qsize.h>
 #include <qvariant.h>
-#include <private/qrhi_p.h>
+#include <rhi/qrhi.h>
+
+#include <mutex>
 
 #include <QDebug>
 
@@ -43,9 +45,11 @@ public:
     int mappedCount = 0;
     QMutex mapMutex;
     QString subtitleText;
-    QVideoFrame::RotationAngle rotationAngle = QVideoFrame::Rotation0;
+    QVideo::RotationAngle rotationAngle = QVideo::Rotation0;
     bool mirrored = false;
     QImage image;
+    std::once_flag imageOnceFlag;
+
 private:
     Q_DISABLE_COPY(QVideoFramePrivate)
 };
@@ -640,7 +644,7 @@ void QVideoFrame::setEndTime(qint64 time)
 void QVideoFrame::setRotationAngle(QVideoFrame::RotationAngle angle)
 {
     if (d)
-        d->rotationAngle = angle;
+        d->rotationAngle = QVideo::RotationAngle(angle);
 }
 
 /*!
@@ -648,7 +652,7 @@ void QVideoFrame::setRotationAngle(QVideoFrame::RotationAngle angle)
  */
 QVideoFrame::RotationAngle QVideoFrame::rotationAngle() const
 {
-    return d ? d->rotationAngle : Rotation0;
+    return QVideoFrame::RotationAngle(d ? d->rotationAngle : QVideo::Rotation0);
 }
 
 /*!
@@ -676,11 +680,12 @@ QImage QVideoFrame::toImage() const
 {
     if (!isValid())
         return {};
-    if (!d->image.isNull())
-        return d->image;
 
-    d->image = qImageFromVideoFrame(*this, rotationAngle(), mirrored(),
-                                    surfaceFormat().scanLineDirection() != QVideoFrameFormat::TopToBottom);
+    std::call_once(d->imageOnceFlag, [this]() {
+        const bool mirrorY = surfaceFormat().scanLineDirection() != QVideoFrameFormat::TopToBottom;
+        d->image = qImageFromVideoFrame(*this, QVideo::RotationAngle(rotationAngle()), mirrored(), mirrorY);
+    });
+
     return d->image;
 }
 
