@@ -14,8 +14,7 @@
 
 #include <d3d11_1.h>
 
-#include <private/qrhi_p.h>
-#include <private/qrhid3d11_p.h>
+#include <rhi/qrhi.h>
 
 #if QT_CONFIG(opengl)
 #  include <qopenglcontext.h>
@@ -25,19 +24,18 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(qLcEvrD3DPresentEngine, "qt.multimedia.evrd3dpresentengine")
+static Q_LOGGING_CATEGORY(qLcEvrD3DPresentEngine, "qt.multimedia.evrd3dpresentengine")
 
 class IMFSampleVideoBuffer: public QAbstractVideoBuffer
 {
 public:
-    IMFSampleVideoBuffer(QWindowsIUPointer<IDirect3DDevice9Ex> device,
-                          IMFSample *sample, QRhi *rhi, QVideoFrame::HandleType type = QVideoFrame::NoHandle)
+    IMFSampleVideoBuffer(ComPtr<IDirect3DDevice9Ex> device,
+                         const ComPtr<IMFSample> &sample, QRhi *rhi, QVideoFrame::HandleType type = QVideoFrame::NoHandle)
         : QAbstractVideoBuffer(type, rhi)
         , m_device(device)
+        , m_sample(sample)
         , m_mapMode(QVideoFrame::NotMapped)
     {
-        sample->AddRef();
-        m_sample.reset(sample);
     }
 
     ~IMFSampleVideoBuffer() override
@@ -59,24 +57,24 @@ public:
                 return {};
 
         } else {
-            QWindowsIUPointer<IMFMediaBuffer> buffer;
-            HRESULT hr = m_sample->GetBufferByIndex(0, buffer.address());
+            ComPtr<IMFMediaBuffer> buffer;
+            HRESULT hr = m_sample->GetBufferByIndex(0, buffer.GetAddressOf());
             if (FAILED(hr))
                 return {};
 
-            QWindowsIUPointer<IDirect3DSurface9> surface;
-            hr = MFGetService(buffer.get(), MR_BUFFER_SERVICE, IID_IDirect3DSurface9, (void **)(surface.address()));
+            ComPtr<IDirect3DSurface9> surface;
+            hr = MFGetService(buffer.Get(), MR_BUFFER_SERVICE, IID_IDirect3DSurface9, (void **)(surface.GetAddressOf()));
             if (FAILED(hr))
                 return {};
 
             if (FAILED(surface->GetDesc(&desc)))
                 return {};
 
-            if (FAILED(m_device->CreateOffscreenPlainSurface(desc.Width, desc.Height, desc.Format, D3DPOOL_SYSTEMMEM, m_memSurface.address(), nullptr)))
+            if (FAILED(m_device->CreateOffscreenPlainSurface(desc.Width, desc.Height, desc.Format, D3DPOOL_SYSTEMMEM, m_memSurface.GetAddressOf(), nullptr)))
                 return {};
 
-            if (FAILED(m_device->GetRenderTargetData(surface.get(), m_memSurface.get()))) {
-                m_memSurface.reset();
+            if (FAILED(m_device->GetRenderTargetData(surface.Get(), m_memSurface.Get()))) {
+                m_memSurface.Reset();
                 return {};
             }
         }
@@ -106,18 +104,18 @@ public:
     }
 
 protected:
-    QWindowsIUPointer<IDirect3DDevice9Ex> m_device;
-    QWindowsIUPointer<IMFSample> m_sample;
+    ComPtr<IDirect3DDevice9Ex> m_device;
+    ComPtr<IMFSample> m_sample;
 
 private:
-    QWindowsIUPointer<IDirect3DSurface9> m_memSurface;
+    ComPtr<IDirect3DSurface9> m_memSurface;
     QVideoFrame::MapMode m_mapMode;
 };
 
 class QVideoFrameD3D11Textures: public QVideoFrameTextures
 {
 public:
-    QVideoFrameD3D11Textures(std::unique_ptr<QRhiTexture> &&tex, QWindowsIUPointer<ID3D11Texture2D> &&d3d11tex)
+    QVideoFrameD3D11Textures(std::unique_ptr<QRhiTexture> &&tex, ComPtr<ID3D11Texture2D> &&d3d11tex)
        : m_tex(std::move(tex))
        , m_d3d11tex(std::move(d3d11tex))
     {}
@@ -129,13 +127,13 @@ public:
 
 private:
     std::unique_ptr<QRhiTexture> m_tex;
-    QWindowsIUPointer<ID3D11Texture2D> m_d3d11tex;
+    ComPtr<ID3D11Texture2D> m_d3d11tex;
 };
 
 class D3D11TextureVideoBuffer: public IMFSampleVideoBuffer
 {
 public:
-    D3D11TextureVideoBuffer(QWindowsIUPointer<IDirect3DDevice9Ex> device, IMFSample *sample,
+    D3D11TextureVideoBuffer(ComPtr<IDirect3DDevice9Ex> device, const ComPtr<IMFSample> &sample,
                             HANDLE sharedHandle, QRhi *rhi)
         : IMFSampleVideoBuffer(std::move(device), sample, rhi, QVideoFrame::RhiTextureHandle)
         , m_sharedHandle(sharedHandle)
@@ -154,8 +152,8 @@ public:
         if (!dev)
             return {};
 
-        QWindowsIUPointer<ID3D11Texture2D> d3d11tex;
-        HRESULT hr = dev->OpenSharedResource(m_sharedHandle, __uuidof(ID3D11Texture2D), (void**)(d3d11tex.address()));
+        ComPtr<ID3D11Texture2D> d3d11tex;
+        HRESULT hr = dev->OpenSharedResource(m_sharedHandle, __uuidof(ID3D11Texture2D), (void**)(d3d11tex.GetAddressOf()));
         if (SUCCEEDED(hr)) {
             D3D11_TEXTURE2D_DESC desc = {};
             d3d11tex->GetDesc(&desc);
@@ -168,7 +166,7 @@ public:
                 return {};
 
             std::unique_ptr<QRhiTexture> tex(rhi->newTexture(format, QSize{int(desc.Width), int(desc.Height)}, 1, {}));
-            tex->createFrom({quint64(d3d11tex.get()), 0});
+            tex->createFrom({quint64(d3d11tex.Get()), 0});
             return std::make_unique<QVideoFrameD3D11Textures>(std::move(tex), std::move(d3d11tex));
 
         } else {
@@ -288,7 +286,7 @@ private:
 class OpenGlVideoBuffer: public IMFSampleVideoBuffer
 {
 public:
-    OpenGlVideoBuffer(QWindowsIUPointer<IDirect3DDevice9Ex> device, IMFSample *sample,
+    OpenGlVideoBuffer(ComPtr<IDirect3DDevice9Ex> device, const ComPtr<IMFSample> &sample,
                       const WglNvDxInterop &wglNvDxInterop, HANDLE sharedHandle, QRhi *rhi)
         : IMFSampleVideoBuffer(std::move(device), sample, rhi, QVideoFrame::RhiTextureHandle)
         , m_sharedHandle(sharedHandle)
@@ -298,29 +296,29 @@ public:
     std::unique_ptr<QVideoFrameTextures> mapTextures(QRhi *rhi) override
     {
         if (!m_texture) {
-            QWindowsIUPointer<IMFMediaBuffer> buffer;
-            HRESULT hr = m_sample->GetBufferByIndex(0, buffer.address());
+            ComPtr<IMFMediaBuffer> buffer;
+            HRESULT hr = m_sample->GetBufferByIndex(0, buffer.GetAddressOf());
             if (FAILED(hr))
                 return {};
 
-            QWindowsIUPointer<IDirect3DSurface9> surface;
-            hr = MFGetService(buffer.get(), MR_BUFFER_SERVICE, IID_IDirect3DSurface9,
-                              (void **)(surface.address()));
+            ComPtr<IDirect3DSurface9> surface;
+            hr = MFGetService(buffer.Get(), MR_BUFFER_SERVICE, IID_IDirect3DSurface9,
+                              (void **)(surface.GetAddressOf()));
             if (FAILED(hr))
                 return {};
 
-            hr = surface->GetContainer(IID_IDirect3DTexture9, (void **)m_texture.address());
+            hr = surface->GetContainer(IID_IDirect3DTexture9, (void **)m_texture.GetAddressOf());
             if (FAILED(hr))
                 return {};
         }
 
-        return QVideoFrameOpenGlTextures::create(m_wgl, rhi, m_device.get(), m_texture.get(), m_sharedHandle);
+        return QVideoFrameOpenGlTextures::create(m_wgl, rhi, m_device.Get(), m_texture.Get(), m_sharedHandle);
     }
 
 private:
     HANDLE m_sharedHandle = nullptr;
     WglNvDxInterop m_wgl;
-    QWindowsIUPointer<IDirect3DTexture9> m_texture;
+    ComPtr<IDirect3DTexture9> m_texture;
 };
 #endif
 
@@ -344,9 +342,9 @@ void D3DPresentEngine::setSink(QVideoSink *sink)
     m_sink = sink;
 
     releaseResources();
-    m_device.reset();
-    m_devices.reset();
-    m_D3D9.reset();
+    m_device.Reset();
+    m_devices.Reset();
+    m_D3D9.Reset();
 
     if (!m_sink)
         return;
@@ -364,10 +362,10 @@ void D3DPresentEngine::setSink(QVideoSink *sink)
 
 HRESULT D3DPresentEngine::initializeD3D()
 {
-    HRESULT hr = Direct3DCreate9Ex(D3D_SDK_VERSION, m_D3D9.address());
+    HRESULT hr = Direct3DCreate9Ex(D3D_SDK_VERSION, m_D3D9.GetAddressOf());
 
     if (SUCCEEDED(hr))
-        hr = DXVA2CreateDirect3DDeviceManager9(&m_deviceResetToken, m_devices.address());
+        hr = DXVA2CreateDirect3DDeviceManager9(&m_deviceResetToken, m_devices.GetAddressOf());
 
     return hr;
 }
@@ -414,7 +412,12 @@ static bool readWglNvDxInteropProc(WglNvDxInterop &f)
 
     HWND hwnd = ::GetShellWindow();
     auto dc = ::GetDC(hwnd);
-    bool hasExtension = strstr(wglGetExtensionsStringARB(dc), "WGL_NV_DX_interop");
+
+    const char *wglExtString = wglGetExtensionsStringARB(dc);
+    if (!wglExtString)
+        qCDebug(qLcEvrD3DPresentEngine) << "WGL extensions missing (wglGetExtensionsStringARB returned null)";
+
+    bool hasExtension = wglExtString && strstr(wglExtString, "WGL_NV_DX_interop");
     ReleaseDC(hwnd, dc);
     if (!hasExtension) {
         qCDebug(qLcEvrD3DPresentEngine) << "WGL_NV_DX_interop missing";
@@ -431,6 +434,20 @@ static bool readWglNvDxInteropProc(WglNvDxInterop &f)
 }
 #endif
 
+namespace {
+
+bool hwTextureRenderingEnabled() {
+    // add possibility for an user to opt-out HW video rendering
+    // using the same env. variable as for FFmpeg backend
+    static bool isDisableConversionSet = false;
+    static const int disableHwConversion = qEnvironmentVariableIntValue(
+            "QT_DISABLE_HW_TEXTURES_CONVERSION", &isDisableConversionSet);
+
+    return !isDisableConversionSet || !disableHwConversion;
+}
+
+}
+
 HRESULT D3DPresentEngine::createD3DDevice()
 {
     if (!m_D3D9 || !m_devices)
@@ -438,23 +455,26 @@ HRESULT D3DPresentEngine::createD3DDevice()
 
     m_useTextureRendering = false;
     UINT adapterID = 0;
-    QRhi *rhi = m_sink ? m_sink->rhi() : nullptr;
-    if (rhi) {
-        if (rhi->backend() == QRhi::D3D11) {
-            m_useTextureRendering = findD3D11AdapterID(*rhi, m_D3D9.get(), adapterID);
-#if QT_CONFIG(opengl)
-        } else if (rhi->backend() == QRhi::OpenGLES2)  {
-            m_useTextureRendering = readWglNvDxInteropProc(m_wglNvDxInterop);
-#endif
-        } else {
-            qCDebug(qLcEvrD3DPresentEngine) << "Not supported RHI backend type";
-        }
-    } else {
-        qCDebug(qLcEvrD3DPresentEngine) << "No RHI associated with this sink";
-    }
 
-    if (!m_useTextureRendering)
-        qCDebug(qLcEvrD3DPresentEngine) << "Could not find compatible RHI adapter, zero copy disabled";
+    if (hwTextureRenderingEnabled()) {
+        QRhi *rhi = m_sink ? m_sink->rhi() : nullptr;
+        if (rhi) {
+            if (rhi->backend() == QRhi::D3D11) {
+                m_useTextureRendering = findD3D11AdapterID(*rhi, m_D3D9.Get(), adapterID);
+#if QT_CONFIG(opengl)
+            } else if (rhi->backend() == QRhi::OpenGLES2)  {
+                m_useTextureRendering = readWglNvDxInteropProc(m_wglNvDxInterop);
+#endif
+            } else {
+                qCDebug(qLcEvrD3DPresentEngine) << "Not supported RHI backend type";
+            }
+        } else {
+            qCDebug(qLcEvrD3DPresentEngine) << "No RHI associated with this sink";
+        }
+
+        if (!m_useTextureRendering)
+            qCDebug(qLcEvrD3DPresentEngine) << "Could not find compatible RHI adapter, zero copy disabled";
+    }
 
     D3DCAPS9 ddCaps;
     ZeroMemory(&ddCaps, sizeof(ddCaps));
@@ -482,7 +502,7 @@ HRESULT D3DPresentEngine::createD3DDevice()
     pp.Flags = D3DPRESENTFLAG_VIDEO;
     pp.PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;
 
-    QWindowsIUPointer<IDirect3DDevice9Ex> device;
+    ComPtr<IDirect3DDevice9Ex> device;
 
     hr = m_D3D9->CreateDeviceEx(
                 adapterID,
@@ -491,7 +511,7 @@ HRESULT D3DPresentEngine::createD3DDevice()
                 vp | D3DCREATE_NOWINDOWCHANGES | D3DCREATE_MULTITHREADED | D3DCREATE_FPU_PRESERVE,
                 &pp,
                 NULL,
-                device.address()
+                device.GetAddressOf()
                 );
     if (FAILED(hr))
         return hr;
@@ -500,7 +520,7 @@ HRESULT D3DPresentEngine::createD3DDevice()
     if (FAILED(hr))
         return hr;
 
-    hr = m_devices->ResetDevice(device.get(), m_deviceResetToken);
+    hr = m_devices->ResetDevice(device.Get(), m_deviceResetToken);
     if (FAILED(hr))
         return hr;
 
@@ -510,7 +530,7 @@ HRESULT D3DPresentEngine::createD3DDevice()
 
 bool D3DPresentEngine::isValid() const
 {
-    return m_device.get() != nullptr;
+    return m_device.Get() != nullptr;
 }
 
 void D3DPresentEngine::releaseResources()
@@ -526,7 +546,7 @@ HRESULT D3DPresentEngine::getService(REFGUID, REFIID riid, void** ppv)
         if (!m_devices) {
             hr = MF_E_UNSUPPORTED_SERVICE;
         } else {
-            *ppv = m_devices.get();
+            *ppv = m_devices.Get();
             m_devices->AddRef();
         }
     } else {
@@ -572,7 +592,9 @@ HRESULT D3DPresentEngine::checkFormat(D3DFORMAT format)
     return ok ? S_OK : D3DERR_NOTAVAILABLE;
 }
 
-HRESULT D3DPresentEngine::createVideoSamples(IMFMediaType *format, QList<IMFSample*> &videoSampleQueue, QSize frameSize)
+HRESULT D3DPresentEngine::createVideoSamples(IMFMediaType *format,
+                                             QList<ComPtr<IMFSample>> &videoSampleQueue,
+                                             QSize frameSize)
 {
     if (!format || !m_device)
         return MF_E_UNEXPECTED;
@@ -604,24 +626,24 @@ HRESULT D3DPresentEngine::createVideoSamples(IMFMediaType *format, QList<IMFSamp
     for (int i = 0; i < PRESENTER_BUFFER_COUNT; i++) {
         // texture ref cnt is increased by GetSurfaceLevel()/MFCreateVideoSampleFromSurface()
         // below, so it will be destroyed only when the sample pool is released.
-        QWindowsIUPointer<IDirect3DTexture9> texture;
+        ComPtr<IDirect3DTexture9> texture;
         HANDLE sharedHandle = nullptr;
-        hr = m_device->CreateTexture(width, height, 1, D3DUSAGE_RENDERTARGET, (D3DFORMAT)d3dFormat,  D3DPOOL_DEFAULT, texture.address(), &sharedHandle);
+        hr = m_device->CreateTexture(width, height, 1, D3DUSAGE_RENDERTARGET, (D3DFORMAT)d3dFormat,  D3DPOOL_DEFAULT, texture.GetAddressOf(), &sharedHandle);
         if (FAILED(hr))
             break;
 
-        QWindowsIUPointer<IDirect3DSurface9> surface;
-        hr = texture->GetSurfaceLevel(0, surface.address());
+        ComPtr<IDirect3DSurface9> surface;
+        hr = texture->GetSurfaceLevel(0, surface.GetAddressOf());
         if (FAILED(hr))
             break;
 
-        QWindowsIUPointer<IMFSample> videoSample;
-        hr = MFCreateVideoSampleFromSurface(surface.get(), videoSample.address());
+        ComPtr<IMFSample> videoSample;
+        hr = MFCreateVideoSampleFromSurface(surface.Get(), videoSample.GetAddressOf());
         if (FAILED(hr))
             break;
 
-        m_sampleTextureHandle[i] = {videoSample.get(), sharedHandle};
-        videoSampleQueue.append(videoSample.release());
+        m_sampleTextureHandle[i] = {videoSample.Get(), sharedHandle};
+        videoSampleQueue.append(videoSample);
     }
 
     if (SUCCEEDED(hr)) {
@@ -633,14 +655,14 @@ HRESULT D3DPresentEngine::createVideoSamples(IMFMediaType *format, QList<IMFSamp
     return hr;
 }
 
-QVideoFrame D3DPresentEngine::makeVideoFrame(IMFSample *sample)
+QVideoFrame D3DPresentEngine::makeVideoFrame(const ComPtr<IMFSample> &sample)
 {
     if (!sample)
         return {};
 
     HANDLE sharedHandle = nullptr;
     for (const auto &p : m_sampleTextureHandle)
-        if (p.first == sample)
+        if (p.first == sample.Get())
             sharedHandle = p.second;
 
     QAbstractVideoBuffer *vb = nullptr;
